﻿Imports System.IO.Ports
Imports System.Management

Module Module1

    Public Class SerialParam
        Private vPortName As String = ""
        Private vBaudRate As Decimal = 115200
        Private vParity As String = "NONE"
        Private vStopBits As Decimal = 1
        Private vDataBits As Integer = 8

        Public Property PortName As String
            Get
                Return vPortName
            End Get
            Set(value As String)
                vPortName = value
            End Set
        End Property

        Public Property BaudRate As Decimal
            Get
                Return vBaudRate
            End Get
            Set(value As Decimal)
                vBaudRate = value
            End Set
        End Property

        Public Property Parity As String
            Get
                Return vParity
            End Get
            Set(value As String)
                vParity = value
            End Set
        End Property

        Public Property StopBits As Decimal
            Get
                Return vStopBits
            End Get
            Set(value As Decimal)
                vStopBits = value
            End Set
        End Property

        Public Property DataBits As Integer
            Get
                Return vDataBits
            End Get
            Set(value As Integer)
                vDataBits = value
            End Set
        End Property
    End Class

    Public oSerialParam As SerialParam = New SerialParam
    Public oForm1 As Form1
    Public oForm2 As Form2
    Public aPorts As String()
    Public bPortFind As Boolean = False
    Public bCancel As Boolean = False

    Public iPosition_X As Integer
    Public iPosition_Y As Integer

    Public iLogInterval As Integer
    Public iAverage As Integer

    ''' <summary>
    ''' アプリケーションのメイン エントリ ポイントです。
    ''' </summary>
    <STAThread> _
    Sub Main()

        'Mutex名を決める（必ずアプリケーション固有の文字列に変更すること！）
        Dim mutexName As String = Application.ProductName
        'Mutex名の先頭に「Global\」を付けて、Global Mutexにする
        mutexName = "Global\" & mutexName

        'すべてのユーザーにフルコントロールを許可するMutexSecurityを作成する
        Dim rule As New System.Security.AccessControl.MutexAccessRule( _
            New System.Security.Principal.SecurityIdentifier( _
                System.Security.Principal.WellKnownSidType.WorldSid, Nothing), _
            System.Security.AccessControl.MutexRights.FullControl, _
            System.Security.AccessControl.AccessControlType.Allow)
        Dim mutexSecurity As New System.Security.AccessControl.MutexSecurity()
        mutexSecurity.AddAccessRule(rule)

        'Mutexオブジェクトを作成する
        Dim createdNew As Boolean
        Dim mutex As New System.Threading.Mutex(False, mutexName, createdNew, mutexSecurity)

        Dim hasHandle As Boolean = False
        Try
            Try
                'ミューテックスの所有権を要求する
                hasHandle = mutex.WaitOne(0, False)
                '.NET Framework 2.0以降の場合
            Catch ex As System.Threading.AbandonedMutexException
                '別のアプリケーションがミューテックスを解放しないで終了した時
                hasHandle = True
            End Try
            'ミューテックスを得られたか調べる
            If hasHandle = False Then
                '得られなかった場合は、すでに起動していると判断して終了
                MessageBox.Show("多重起動はできません。")
                Return
            End If

            'はじめからMainメソッドにあったコードを実行
            Application.EnableVisualStyles()
            Application.SetCompatibleTextRenderingDefault(False)
            AppStart()

        Finally
            If hasHandle Then
                'ミューテックスを解放する
                mutex.ReleaseMutex()
            End If
            mutex.Close()
        End Try
    End Sub

    Public Sub AppStart()

        iPosition_X = My.Settings.Position_X
        iPosition_Y = My.Settings.Position_Y
        iLogInterval = My.Settings.LogInterval
        iAverage = My.Settings.Average

        oSerialParam.PortName = My.Settings.PortName
        oSerialParam.BaudRate = My.Settings.BaudRate
        oSerialParam.Parity = My.Settings.Parity
        oSerialParam.StopBits = My.Settings.StopBits
        oSerialParam.DataBits = My.Settings.DataBits

        aPorts = GetSerialPortList()
        If Not aPorts Is Nothing Then
            If aPorts.Length > 0 Then
                bPortFind = True
            End If
        End If

        If bPortFind Then
            If oSerialParam.PortName Is Nothing Then
                bPortFind = False
                oForm2 = New Form2
                oForm2.ShowDialog()
            Else
                Dim bPort As Boolean = False
                Dim Port As String
                For Each Port In aPorts
                    Dim PortName As String() = Port.Split(";")
                    If PortName(1) = oSerialParam.PortName Then
                        bPort = True
                        Exit For
                    End If
                Next
                If bPort = False Then
                    bCancel = True
                    bPortFind = False
                    oForm2 = New Form2
                    oForm2.ShowDialog()
                    If bCancel Then
                        MsgBox("接続可能なCOMポート機器が指定されませんでした。" + vbCr + "アプリケーションを終了します。", MsgBoxStyle.OkOnly + MsgBoxStyle.Critical, "接続機器エラー")
                        End
                    End If
                End If
            End If
        End If

        If bPortFind = False Then
            MsgBox("接続可能なCOMポート機器が有りません。" + vbCr + "アプリケーションを終了します。", MsgBoxStyle.OkOnly + MsgBoxStyle.Critical, "接続機器エラー")
            End
        Else
            oForm1 = New Form1
            oForm1.ShowDialog()
        End If

    End Sub

    Public Function GetSerialPortList() As String()

        Dim portList As String = ""
        Dim ports As String() = SerialPort.GetPortNames()
        If ports.Length < 1 Then
            Return Nothing
        End If

        Dim mcPnPEntity As ManagementClass = New ManagementClass("Win32_PnPEntity")
        Dim manageObjCol As ManagementObjectCollection = mcPnPEntity.GetInstances()
        Dim manageObj As ManagementObject

        For Each manageObj In manageObjCol
            Dim namePropertyVal = manageObj.GetPropertyValue("Name")
            If Not namePropertyVal Is Nothing Then
                Dim name As String = namePropertyVal.ToString
                Dim port As String

                For Each port In ports
                    Dim check = New System.Text.RegularExpressions.Regex("(" + port + ")")
                    If check.IsMatch(name) Then
                        If portList.Length > 0 Then
                            portList += ","
                        End If
                        portList += name + ";" + port
                    End If
                Next
            End If
        Next

        If portList.Length > 0 Then
            ports = portList.Split(",")
        Else
            ports = Nothing
        End If

        Return ports

    End Function


End Module
